@@ -33,6 +33,10 @@ NULL
 #' @param x a partition object
 #' @param verbose logical
 #' @param cpos ...
+#' @param html ...
+#' @param highlight ... 
+#' @param cqp ...
+#' @param tooltips ...
 #' @param meta ...
 #' @param cutoff maximum number of tokens to decode
 #' @param ... further parameters
@@ -201,4 +205,5 @@ setAs("partition", "data.table", function(from) data.table(count(from)) )
 #' @rdname partition_class
 setMethod("hist", "partition", function(x, ...){hist(x@stat[,"count"], ...)})
 
-
+#' @rdname partition_class
+setMethod("length", "partition", function(x) 1)
